/*******************************************************************************
 * Copyright (c) 2011 Ericsson
 * 
 * All rights reserved. This program and the accompanying materials are
 * made available under the terms of the Eclipse Public License v1.0 which
 * accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Bernd Hufmann - Initial API and implementation
 *******************************************************************************/
package org.eclipse.linuxtools.tmf.reference.ui.trace;

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.linuxtools.tmf.event.TmfEvent;
import org.eclipse.linuxtools.tmf.experiment.TmfExperiment;
import org.eclipse.linuxtools.tmf.parser.ITmfEventParser;
import org.eclipse.linuxtools.tmf.reference.ui.Activator;
import org.eclipse.linuxtools.tmf.signal.TmfExperimentSelectedSignal;
import org.eclipse.linuxtools.tmf.trace.ITmfTrace;

/**
 *  Singleton class to facilitate the test cases. Creates UML2SD view and loader objects as well as provides
 *  utility methods for interacting with the loader/view.  
 */
public class Uml2SDTestFacility {
    
    // ------------------------------------------------------------------------
    // Attributes
    // ------------------------------------------------------------------------
    private static Uml2SDTestFacility fInstance = null;
    
    private TmfTraceStub fTrace = null;
    private TmfUml2SDTestTrace    fParser = null;
    private TmfExperiment<TmfEvent> fExperiment = null;
    
    private boolean fIsInitialized = false;
    
    // ------------------------------------------------------------------------
    // Constructors
    // ------------------------------------------------------------------------
    private Uml2SDTestFacility() {
    }

    // ------------------------------------------------------------------------
    // Operations
    // ------------------------------------------------------------------------
    public static Uml2SDTestFacility getInstance() {
        if (fInstance == null) {
            fInstance = new Uml2SDTestFacility();
        }
        return fInstance;
    }

    /**
     * Initial the test facility.
     * @param isGlobal true if multiple test cases are run in sequence (AllTests) or false for a single test case
     */
    public void init() {
        
        if (!fIsInitialized) {

            fParser = new TmfUml2SDTestTrace();
            fTrace = setupTrace(fParser);
            
            fIsInitialized = true;
        }
    }

    
    private TmfTraceStub setupTrace(ITmfEventParser parser) {
        
            try {
                // Create test trace object
                URL location = FileLocator.find(Activator.getDefault().getBundle(), new Path("tracesets/sdEvents"), null); //$NON-NLS-1$
                File test = new File(FileLocator.toFileURL(location).toURI());
                return new TmfTraceStub(test.getPath(), 500, true, parser);
            } catch (URISyntaxException e) {
                e.printStackTrace();
                throw new RuntimeException(e);
            } catch (IOException e) {
                e.printStackTrace();
                throw new RuntimeException(e);
            }
    }
    
    /**
     * Dispose the resource
     */
    public void dispose() {
        if (fIsInitialized) {
            fExperiment.dispose();
            fIsInitialized = false;
        }
    }
    
    /**
     * @return current experiment.
     */
    public TmfExperiment<TmfEvent> getExperiment() {
        return fExperiment;
    }
    
    /**
     * Selects the experiment. 
     * @param wait true to wait for indexing to finish else false
     */
    public void selectExperiment() {
        fTrace = setupTrace(fParser);

        ITmfTrace traces[] = new ITmfTrace[1];
        traces[0] = fTrace;
        fExperiment = new TmfExperiment<TmfEvent>(TmfEvent.class, "TestExperiment", traces); //$NON-NLS-1$
        fTrace.broadcast(new TmfExperimentSelectedSignal<TmfEvent>(this, fExperiment));
    }
    
    /**
     * Disposes the experiment.
     */
    public void disposeExperiment() {
        fExperiment.dispose();
    }
    
}
